#!/ramdisk/bin/perl

use strict;


BEGIN {
    my $homedir = ( getpwuid($>) )[7];
    my @user_include;
    foreach my $path (@INC) {
        if ( -d $homedir . '/perl' . $path ) {
            push @user_include, $homedir . '/perl' . $path;
        }
    }
    unshift @INC, @user_include;
}


use CGI;
use XML::DOM::XPath;
use Template;
use Data::Dumper;

my $cgi = new CGI;

print $cgi->header();

our $OUT_DIR = "/home1/investq6/tmp/parsing/";
#our $OUT_DIR = "/tmp/sites/idna/parsing/";


my $active_date = get_latest_date();
if ($cgi->param('date')) {
	$active_date = $cgi->param('date');
}	
my $active_dir = $OUT_DIR . $active_date;

print STDERR "Setting active dir to $active_dir\n";

#create the new consolidated form
do_updates($active_dir, $cgi); 

#get any new article info
consolidate_articles($active_dir);

#get the data for the form
my %data = ();
$data{articles} = parse_consolidated($active_dir);
$data{selectedDate} = $active_date;
$data{dates} = read_dirs();

#print the form
write_template(\%data);

##################################################################

sub get_latest_date {
	my $dirs = read_dirs();
	return $dirs->[-1];
}	

##################################################################

sub read_dirs {
	opendir(DIR, $OUT_DIR) || die "can't opendir $OUT_DIR: $!";
	my @dirs = grep { /\d{8}/ } readdir(DIR);
      closedir DIR;
	return \@dirs;
}

##################################################################

sub get_file_list {
	my $dir = shift;
	
	opendir(DIR, $dir) || die "can't opendir $OUT_DIR: $!";
	my @xml_files = grep { /.*?\.xml/ && $_ !~ /consolidated/ } readdir(DIR);
  closedir DIR;
	return \@xml_files;
}


##################################################################

sub consolidate_articles {
	my $dir = shift;
	
	my $consolidated = "";
	if (-e $dir . "/consolidated.xml") {
		my $parser= XML::DOM::Parser->new();
	  my $doc = $parser->parsefile ($dir . "/consolidated.xml");
	  my @nodes = $doc->findnodes( '//entry');
	  foreach my $node (@nodes) {
	  	$consolidated .= $node->toString() ."\n";
	  }	
	}	
	$consolidated =~ s!</entries>!!;
	foreach my $file (@{get_file_list($dir)}) {
		my ($source) = $file =~ /(.*?)\.xml/;
		#add data if not already in consolidated
		if ($consolidated !~ m!<source>$source</source>!) {
			print STDERR "editor.cgi::Loading $file to consolidated\n";
			my $line = 0;
			open (IN, $dir . "/" . $file);
			while (<IN>) {
				if ($line > 1 && $_ !~ m!</entries>!) {
					$consolidated .= $_;
				}	
				$line += 1;
			}	
		}	
		else {
			print STDERR "editor.cgi::Ignoring $file as already loaded\n";
		}	
	}
	
	open (OUT, ">$dir/consolidated.xml");
	print OUT qq(<?xml version="1.0" encoding="UTF-8"?>\n);
	print OUT "<entries>\n";
	print OUT $consolidated;
	print OUT "</entries>\n";
	close OUT;
}	

##################################################################

sub get_consolidated_file {
	my $dir = shift;
	my $consolidated = "";
	if (-e $dir . "/consolidated.xml") {
		open IN, $dir . "/consolidated.xml";
		undef $/;
		$consolidated = <IN>;
		close IN;
	}
	return $consolidated;
}		 		

##################################################################

sub parse_consolidated {
	my $dir = shift;

	my $parser= XML::DOM::Parser->new();
  my $doc = $parser->parsefile ($dir . "/consolidated.xml");

  my @nodes = $doc->findnodes( '//entry');
  my @articles = ();
  my $id = 1;

  foreach my $node (@nodes) {
		my %article = ();
		$article{id} = $id;
		
		for my $kid ($node->getChildNodes) {
			my $name = $kid->getNodeName();
			if ($name eq "source" && $kid->getFirstChild()) {
				$article{source} = $kid->getFirstChild()->getNodeValue();
			}  	
			elsif ($name eq "header" && $kid->getFirstChild()) {
				$article{header} = $kid->getFirstChild()->getNodeValue();
			}  	
			elsif ($name eq "link" && $kid->getFirstChild()) {
				$article{link} = $kid->getFirstChild()->getNodeValue();
			}  	
			elsif ($name eq "text" && $kid->getFirstChild()) {
				print STDERR $article{header} . "\n";
				$article{text} = trim($kid->getFirstChild()->getNodeValue());
			}  	
		}
		
		push @articles, \%article;
		$id++;
	}		

	return \@articles;
}

##################################################################
	
sub do_updates {
	my $dir = shift;
	my $cgi = shift;
	
	return unless $cgi->param("submitted");
	
  open (OUT, ">" . $dir . "/consolidated.xml");
	print OUT qq(<?xml version="1.0" encoding="UTF-8"?>\n);
	print OUT "<entries>\n";
  
  my $id = 1;
  my @entries = ();
  while ($cgi->param("order$id")) {
  	my %entry = ();
  	if (!$cgi->param("delete$id")) {
  		$entry{link} = $cgi->param("link$id");
  		$entry{order} = $cgi->param("order$id");
  		$entry{text} = $cgi->param("order$id");
  		$entry{header} = $cgi->param("header$id");
  		$entry{source} = $cgi->param("source$id");
  		push @entries, \%entry;
  	}
	}  		 	
	
	#sort based on order
	sort {$a->{order} <=> $b->{order}} @entries;
	
	foreach my $entry (@entries) {
		print OUT qq(<entry>\n);
		print OUT qq(  <source>$entry->{source}</source>\n);
		print OUT qq(  <header><![CDATA[$entry->{header}]]></header>\n);
		print OUT qq(  <link><![CDATA[$entry->{link}]]></link>\n);
		print OUT qq(  <text><![CDATA[$entry->{text}]]></text>\n);
		print OUT qq(</entry>\n);		
	}
	
	close OUT;	
}



##################################################################

sub trim {
	my $str = shift;
	$str =~ s/^\s+//;
	$str =~ s/\s+$//;
	return $str;
}

##################################################################

sub write_template {
	my $data = shift;
	my $tt = Template->new({
    INTERPOLATE  => 1,
	}) || die "$Template::ERROR\n";

	$tt->process('editor.tmpl', $data) || die $tt->error(), "\n";	
}	